#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <../mod5270/system/sim5270.h>
#include <cfinter.h>

/* The extern directive is only needed if you are creating this code in a .cpp  
   file; it prevents name mangling. If you are crating a .c file, do not include 
   the extern section. */
extern "C" {
                void NMI_C_Part();   // The part of the ISR written in C/C++
                void NMI_ASM_Part(); // The part of the ISR written in Assembly
           }

/* The "Function_Holder()" is just a place holder so we can create some inline 
assembly language code. It will never be called. Note that in 
the __asm__ statements, you need a leading space between the (" and the
 instruction. However, a space is not used for labels. */

void Function_Holder()
{
   __asm__  (" .global  NMI_ASM_Part"); // export the label for ASM part
   __asm__  (" .extern  NMI_C_Part");   // label for the C part of the ISR
   __asm__  ("NMI_ASM_Part:");          // label for the assembly part of the ISR
   __asm__  (" move.w #0x2700,%sr ");   // set the IRQ mask to mask all 
   __asm__  (" lea -60(%a7),%a7 ");     // make space on the system stack
   __asm__  (" movem.l  %d0-%d7/%a0-%a6,(%a7) "); // save all registers
   __asm__  (" jsr NMI_C_Part  ");  
   __asm__  (" movem.l  (%a7),%d0-%d7/%a0-%a6 ");
   __asm__  (" lea 60(%a7),%a7 ");
   __asm__  (" rte");
}

DWORD isr_count=0; /* Count how many times the switch was hit */

/* This is the C/C++ part of the ISR that is called from the assembly code */
void NMI_C_Part() 
{
  /* WARNING WARNING WARNING: No RTOS functions can be called from 
  within an interrupt service routine. */
  sim.eport.epfr=0x02; /* Clear the interrupt edge 0 0 0 0 0 0 1 0 */
  isr_count++;
}

extern "C" 
{
   /* An interrupt setup helper defined in bsp.c 
    This function sets up the 5270 interrupt controller */	
   void SetIntc(long func, int vector, int level, int prio );
   void UserMain(void * pd);
   void putdisp(unsigned short w);
}

/* Declare our interrupt procedure.... */
INTERRUPT(out_irq1_pin_isr, 0x2700 ) { }

/* Helper function to display a decimal number */
void PutDispDecimal(WORD val, BOOL blank_zero )
{
	WORD w;
	
	w = (val / 1000) * 0x1000;
	if ((w == 0) && (blank_zero)) w = 0xF000;
	w += ((val / 100)% 10) * 0x0100;
	if (w == 0xF000) w = 0xFF00;
	w += ((val / 10)% 10) * 0x0010;
	if (w == 0xFF00) w = 0xFFF0;
	w += ((val )% 10) * 0x0001;
	putdisp(w);
}
  
void UserMain(void * pd)
{
  InitializeStack();
  if (EthernetIP==0)GetDHCPAddress(); 
  OSChangePrio(MAIN_PRIO);
  EnableAutoUpdate();

  /* First set up the Eport module to use IRQ1 as a falling-edge active  IRQ pin   
  (See the 5270 UM chapter 15). Set the pin assignment register irq1 pin falling 
  edge sensitive*/ 
  sim.eport.eppar=0x0008; /* 00 00 00 00 00 00 10 00  see table 
                            15-3 in UM */
  sim.eport.epddr=0x0; /* All edge port pins as inputs */
  sim.eport.epier = 0x0002; /* Enable IRQ1 only 0 0 0 0 0 0 1 0 */
  
  /* In the SetIntc( ) call, use the name "NMI_ASM_Part" to set the function 
  address for the "vector" parameter..  */
  SetIntc(
    (long)&NMI_ASM_Part,
     1, /* vector number*/ 
     1, /* Set this to priority 1 but any value from 1 to 6 would  
           be valid.*/
     1 /* The priority within the gross levels; see chapter 10, any 
          value from 0 to 7 is ok */
     ); 

  iprintf("Application started\n");
  iprintf("Press the IRQ button on the development board.\n");
  iprintf("LED will display # of times IRQ button was pressed.\n");
  while (1)
  {
    OSTimeDly(1);
    PutDispDecimal(isr_count,true);
  }
}

